@@ -27,6 +27,7 @@ class NetworkBuilder(DefaultNetworkHandler):
     populations = {}
     projections = {}
     projection_syns = {}
+    projection_types = {}
     projection_syns_pre = {}
     input_lists = {}
     
@@ -54,19 +55,23 @@ class NetworkBuilder(DefaultNetworkHandler):
         self.nml_doc.networks.append(self.network)
         if notes and len(notes)>0:
             self.network.notes = notes
-        if temperature:
+        if temperature is not None:
             self.network.temperature = temperature
             self.network.type = "networkWithTemperature"
    
     #
     #  Overridden from DefaultNetworkHandler
     #    
-    def handle_population(self, population_id, component, size, component_obj=None, properties={}):
+    def handle_population(self, population_id, component, size, component_obj=None, properties={}, notes=None):
       
         if component_obj:
             self.nml_doc.append(component_obj)
         
         pop = neuroml.Population(id=population_id, component=component, size=size)
+        
+        if notes and len(notes)>0:
+            pop.notes = notes
+        
         self.populations[population_id] = pop
         self.network.populations.append(pop)
         for p in properties:
@@ -85,14 +90,17 @@ class NetworkBuilder(DefaultNetworkHandler):
     #  Overridden from DefaultNetworkHandler
     #    
     def handle_location(self, id, population_id, component, x, y, z):
+        ##print('Loc: %s %s (%s,%s,%s)'%(id, population_id, x, y, z))
         self.print_location_information(id, population_id, component, x, y, z)
         
-        if x and y and z:
+        if x is not None and y is not None and z is not None:
             inst = neuroml.Instance(id=id)
 
             inst.location = neuroml.Location(x=x,y=y,z=z)
             self.populations[population_id].instances.append(inst)
             self.populations[population_id].type = 'populationList'
+        else:
+            self.log.warning('Ignoring location: %s %s (%s,%s,%s)'%(id, population_id, x, y, z))
           
     #
     #  Overridden from DefaultNetworkHandler
@@ -116,10 +124,20 @@ class NetworkBuilder(DefaultNetworkHandler):
         elif type=="continuousProjection":
             proj = neuroml.ContinuousProjection(id=id, presynaptic_population=prePop, postsynaptic_population=postPop)
             self.network.continuous_projections.append(proj)
-            self.projection_syns[id] = synapse_obj.id
+            
+            if synapse_obj!=None:
+                self.projection_syns[id] = synapse_obj.id
+            else:
+                self.projection_syns[id] = synapse
+            
+            if pre_synapse_obj == None:
+                pre_synapse_obj = neuroml.SilentSynapse(id="silentSyn_%s"%id)
+                self.nml_doc.silent_synapses.append(pre_synapse_obj)
+                
             self.projection_syns_pre[id] = pre_synapse_obj.id
             
         self.projections[id] = proj
+        self.projection_types[id] = type
         self.weightDelays[id] = hasWeights or hasDelays
 
         self.log.debug("Projection: %s (%s) from %s to %s with syn: %s, weights: %s, delays: %s"%(id, type, prePop, postPop, synapse, hasWeights, hasDelays))
@@ -127,11 +145,15 @@ class NetworkBuilder(DefaultNetworkHandler):
     #
     #  Overridden from DefaultNetworkHandler
     #
-    def finalise_projection(self, id, prePop, postPop, synapse=None, type="projection"):
+    def finalise_projection(self, id, prePop, postPop, synapse=None, type=None):
         '''
             Check whether handle_projection was not called, e.g. due to no connections present
         '''
-        self.log.debug("Projection: %s from %s to %s completed"%(id,prePop,postPop))
+        if type==None:
+            type=self.projection_types[id]
+            
+        self.log.debug("Projection: %s (%s) from %s to %s completed"%(id,type,prePop,postPop))
+        
         if type=="projection":
             present = False
             for p in self.network.projections:
@@ -164,6 +186,14 @@ class NetworkBuilder(DefaultNetworkHandler):
         
         #self.printConnectionInformation(proj_id, conn_id, prePop, postPop, synapseType, preCellId, postCellId, weight)
         
+        pre_cell_path = "../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component)
+        if self.populations[prePop].type==None:
+            pre_cell_path = "../%s[%i]"%(prePop,preCellId)
+            
+        post_cell_path = "../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component)
+        if self.populations[postPop].type==None:
+            post_cell_path = "../%s[%i]"%(postPop,postCellId)
+        
         if isinstance(self.projections[proj_id], neuroml.ElectricalProjection):
             
             instances = False
@@ -185,10 +215,10 @@ class NetworkBuilder(DefaultNetworkHandler):
             else:
                 if weight==1:
                     conn = neuroml.ElectricalConnectionInstance(id=conn_id, \
-                                        pre_cell="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                        pre_cell=pre_cell_path, \
                                         pre_segment=preSegId, \
                                         pre_fraction_along=preFract,
-                                        post_cell="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                        post_cell=post_cell_path, \
                                         post_segment=postSegId,
                                         post_fraction_along=postFract,
                                         synapse=self.projection_syns[proj_id])
@@ -196,10 +226,10 @@ class NetworkBuilder(DefaultNetworkHandler):
                     self.projections[proj_id].electrical_connection_instances.append(conn)
                 else:
                     conn = neuroml.ElectricalConnectionInstanceW(id=conn_id, \
-                                        pre_cell="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                        pre_cell=pre_cell_path, \
                                         pre_segment=preSegId, \
                                         pre_fraction_along=preFract,
-                                        post_cell="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                        post_cell=post_cell_path, \
                                         post_segment=postSegId,
                                         post_fraction_along=postFract,
                                         synapse=self.projection_syns[proj_id],
@@ -215,6 +245,9 @@ class NetworkBuilder(DefaultNetworkHandler):
                 instances = True
                 
             if not instances:
+                if weight!=1:
+                    raise Exception("Case not (yet) supported: weight!=1 when not an instance based population...")
+                
                 conn = neuroml.ContinuousConnection(id=conn_id, \
                                     pre_cell="%s"%(preCellId), \
                                     pre_segment=preSegId, \
@@ -230,10 +263,10 @@ class NetworkBuilder(DefaultNetworkHandler):
             else:
                 if weight==1:
                     conn = neuroml.ContinuousConnectionInstance(id=conn_id, \
-                                        pre_cell="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                        pre_cell=pre_cell_path, \
                                         pre_segment=preSegId, \
                                         pre_fraction_along=preFract,
-                                        post_cell="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                        post_cell=post_cell_path, \
                                         post_segment=postSegId,
                                         post_fraction_along=postFract,
                                         pre_component=self.projection_syns_pre[proj_id],
@@ -242,10 +275,10 @@ class NetworkBuilder(DefaultNetworkHandler):
                     self.projections[proj_id].continuous_connection_instances.append(conn)
                 else:
                     conn = neuroml.ContinuousConnectionInstanceW(id=conn_id, \
-                                        pre_cell="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                        pre_cell=pre_cell_path, \
                                         pre_segment=preSegId, \
                                         pre_fraction_along=preFract,
-                                        post_cell="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                        post_cell=post_cell_path, \
                                         post_segment=postSegId,
                                         post_fraction_along=postFract,
                                         pre_component=self.projection_syns_pre[proj_id],
@@ -259,10 +292,10 @@ class NetworkBuilder(DefaultNetworkHandler):
             if not self.weightDelays[proj_id] and delay==0 and weight==1:
 
                 connection = neuroml.Connection(id=conn_id, \
-                                    pre_cell_id="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                    pre_cell_id=pre_cell_path, \
                                     pre_segment_id=preSegId, \
                                     pre_fraction_along=preFract,
-                                    post_cell_id="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                    post_cell_id=post_cell_path, \
                                     post_segment_id=postSegId,
                                     post_fraction_along=postFract)
 
@@ -270,10 +303,10 @@ class NetworkBuilder(DefaultNetworkHandler):
 
             else:
                 connection = neuroml.ConnectionWD(id=conn_id, \
-                                    pre_cell_id="../%s/%i/%s"%(prePop,preCellId,self.populations[prePop].component), \
+                                    pre_cell_id=pre_cell_path, \
                                     pre_segment_id=preSegId, \
                                     pre_fraction_along=preFract,
-                                    post_cell_id="../%s/%i/%s"%(postPop,postCellId,self.populations[postPop].component), \
+                                    post_cell_id=post_cell_path, \
                                     post_segment_id=postSegId,
                                     post_fraction_along=postFract,
                                     weight=weight,
@@ -304,10 +337,14 @@ class NetworkBuilder(DefaultNetworkHandler):
     def handle_single_input(self, inputListId, id, cellId, segId = 0, fract = 0.5, weight = 1.0):
 
         input_list = self.input_lists[inputListId]
+        target_path = "../%s/%i/%s"%(input_list.populations, cellId, self.populations[input_list.populations].component)
         
+        if self.populations[input_list.populations].type==None:
+            target_path = "../%s[%i]"%(input_list.populations, cellId)
+            
         if weight==1:
             input = neuroml.Input(id=id, 
-                      target="../%s/%i/%s"%(input_list.populations, cellId, self.populations[input_list.populations].component), 
+                      target=target_path, 
                       destination="synapses")  
             if segId!=0:
                 input.segment_id="%s"%(segId)
@@ -316,7 +353,7 @@ class NetworkBuilder(DefaultNetworkHandler):
             input_list.input.append(input)
         else:
             input_w = neuroml.InputW(id=id, 
-                      target="../%s/%i/%s"%(input_list.populations, cellId, self.populations[input_list.populations].component), 
+                      target=target_path, 
                       destination="synapses")  
             if segId!=0:
                 input_w.segment_id="%s"%(segId)
