@@ -16,6 +16,7 @@
   
 import logging
 import sys
+import inspect
 
 import tables   # pytables for HDF5 support
 import numpy as np  
@@ -34,6 +35,7 @@ from neuroml.hdf5.NetworkContainer import InputsList
 
 from neuroml.loaders import read_neuroml2_string
 from neuroml.utils import add_all_to_document
+import os.path
 
 
 class NeuroMLHdf5Parser():
@@ -61,6 +63,32 @@ class NeuroMLHdf5Parser():
     self.netHandler = netHandler
     self.optimized = optimized
     
+    if not self.optimized:
+        # For continued use with old API
+        if not hasattr(self.netHandler,'handle_network') or hasattr(self.netHandler,'handleNetwork'):
+            self.netHandler.handle_network = self.netHandler.handleNetwork
+        if not hasattr(self.netHandler,'handle_document_start') or hasattr(self.netHandler,'handleDocumentStart'):
+            self.netHandler.handle_document_start = self.netHandler.handleDocumentStart
+        if not hasattr(self.netHandler,'handle_population') or hasattr(self.netHandler,'handlePopulation'):
+            self.netHandler.handle_population = self.netHandler.handlePopulation
+        if not hasattr(self.netHandler,'handle_location') or hasattr(self.netHandler,'handleLocation'):
+            self.netHandler.handle_location = self.netHandler.handleLocation
+
+        if not hasattr(self.netHandler,'handle_projection') or hasattr(self.netHandler,'handleProjection'):
+            self.netHandler.handle_projection = self.netHandler.handleProjection
+        if not hasattr(self.netHandler,'finalise_projection') or hasattr(self.netHandler,'finaliseProjection'):
+            self.netHandler.finalise_projection = self.netHandler.finaliseProjection
+
+        if not hasattr(self.netHandler,'handle_connection') or hasattr(self.netHandler,'handleConnection'):
+            self.netHandler.handle_connection = self.netHandler.handleConnection
+        if not hasattr(self.netHandler,'handle_input_list') or hasattr(self.netHandler,'handleInputList'):
+            self.netHandler.handle_input_list = self.netHandler.handleInputList
+        if not hasattr(self.netHandler,'handle_single_input') or hasattr(self.netHandler,'handleSingleInput'):
+            self.netHandler.handle_single_input = self.netHandler.handleSingleInput
+        if not hasattr(self.netHandler,'finalise_input_source') or hasattr(self.netHandler,'finaliseInputSource'):
+            self.netHandler.finalise_input_source = self.netHandler.finaliseInputSource
+    
+    
     
   def parse(self, filename):
       
@@ -71,12 +99,16 @@ class NeuroMLHdf5Parser():
     if hasattr(h5file.root.neuroml._v_attrs,"neuroml_top_level"):
         nml = get_str_attribute_group(h5file.root.neuroml,"neuroml_top_level")
         
-        if sys.version_info[0] == 3:
-            nml = nml.encode()
-            
-        self.nml_doc_extra_elements = read_neuroml2_string(nml, include_includes=True, verbose=False)
-        
-        #print(self.nml_doc_extra_elements.summary())
+        try:
+            self.nml_doc_extra_elements = read_neuroml2_string(nml, 
+                                                               include_includes=True, 
+                                                               verbose=False,
+                                                               base_path=os.path.dirname(os.path.abspath(filename)))
+        except Exception as e:
+            print('Unable to read XML string extracted from HDF5 file!')
+            print(e)
+            print(nml)
+            raise e
         
         self.log.info("Extracted NeuroML2 elements from extra string found in HDF5 file")
     
@@ -238,6 +270,8 @@ class NeuroMLHdf5Parser():
 
             for attrName in d.attrs._v_attrnames:
                 val = d.attrs.__getattr__(attrName)
+                if not isinstance(val, str):
+                    val = val.decode()
 
                 #self.log.debug("Val of attribute: "+ attrName + " is "+ str(val))
 
@@ -262,7 +296,8 @@ class NeuroMLHdf5Parser():
 
             if self.nml_doc_extra_elements:
                 synapse_obj = self.nml_doc_extra_elements.get_by_id(self.currentSynapse)
-                pre_synapse_obj = self.nml_doc_extra_elements.get_by_id(self.currentPreSynapse)
+                
+                pre_synapse_obj = self.nml_doc_extra_elements.get_by_id(self.currentPreSynapse) if len(self.currentPreSynapse)>0 else None
                 
             else:
                 synapse_obj = None
@@ -288,7 +323,7 @@ class NeuroMLHdf5Parser():
                 row = d[i,:]
 
                 id =  int(row[indexId]) if indexId>0 else i
-
+                
                 preCellId =  int(row[indexPreCellId])
 
                 if indexPreSegId >= 0:
@@ -354,6 +389,8 @@ class NeuroMLHdf5Parser():
 
             for attrName in d.attrs._v_attrnames:
                 val = d.attrs.__getattr__(attrName)
+                if not isinstance(val, str):
+                    val = val.decode()
 
                 self.log.debug("Val of attribute: "+ attrName + " is "+ str(val))
 
@@ -394,6 +431,9 @@ class NeuroMLHdf5Parser():
                                             segId = segId,         
                                             fract = fractAlong,
                                             weight = weight) 
+                                        
+            self.netHandler.finalise_input_source(self.currInputList)
+            
         else:
             
             #TODO: a better way to convert???
@@ -407,10 +447,10 @@ class NeuroMLHdf5Parser():
         for node in g:
             if self._is_dataset(node) and node.name == name:
               size = node.shape[0]
-
+              
         if size == -1:
             size = g._v_attrs.size
-            
+              
         return size
     
     
@@ -440,12 +480,8 @@ class NeuroMLHdf5Parser():
     
     if g._v_name.count('population_')>=1:
         # TODO: a better check to see if the attribute is a str or numpy.ndarray
-        self.currPopulation = g._v_attrs.id[0]
-        if (len(self.currPopulation)==1):          # i.e. it was a str and just took the first letter...
-          self.currPopulation = g._v_attrs.id
-        self.currentComponent = g._v_attrs.component[0]
-        if (len(self.currentComponent)==1):
-          self.currentComponent = g._v_attrs.component
+        self.currPopulation = get_str_attribute_group(g,'id')
+        self.currentComponent = get_str_attribute_group(g,'component')
         
         size = self._get_node_size(g,self.currPopulation)
         
@@ -455,7 +491,8 @@ class NeuroMLHdf5Parser():
                 name = str(fname).split(':')[1]
                 properties[name] = get_str_attribute_group(g,fname)
         
-        self.log.debug("Found a population: "+ self.currPopulation+", component: "+self.currentComponent+", size: "+ str(size)+", properties: %s"%properties)
+        self.log.debug("Found a population: %s, component: %s, size: %s, properties: %s" % \
+                       (self.currPopulation,self.currentComponent,size,properties))
         
         if not self.optimized:
             if self.nml_doc_extra_elements:
@@ -463,7 +500,10 @@ class NeuroMLHdf5Parser():
             else:
                 component_obj = None
 
-            self.netHandler.handle_population(self.currPopulation, self.currentComponent, size, component_obj=component_obj, properties=properties)
+            if 'properties' in inspect.getargspec(self.netHandler.handle_population)[0]:
+                self.netHandler.handle_population(self.currPopulation, self.currentComponent, size, component_obj=component_obj, properties=properties)
+            else:
+                self.netHandler.handle_population(self.currPopulation, self.currentComponent, size, component_obj=component_obj)
         else:
             self.currOptPopulation = PopulationContainer(id=self.currPopulation, component=self.currentComponent, size=size)
             
@@ -474,22 +514,23 @@ class NeuroMLHdf5Parser():
         
     if g._v_name.count('projection_')>=1:
       
-        self.currentProjectionId = g._v_attrs.id
-        self.currentProjectionType = g._v_attrs.type if g._v_attrs.type else "projection"
-        self.currentProjectionPrePop = g._v_attrs.presynapticPopulation
-        self.currentProjectionPostPop = g._v_attrs.postsynapticPopulation
+        self.currentProjectionId = get_str_attribute_group(g,'id')
+        pt = get_str_attribute_group(g,'type')
+        self.currentProjectionType = pt if pt else "projection"
+        self.currentProjectionPrePop = get_str_attribute_group(g,'presynapticPopulation')
+        self.currentProjectionPostPop = get_str_attribute_group(g,'postsynapticPopulation')
+        
         if "synapse" in g._v_attrs:
-            self.currentSynapse = g._v_attrs.synapse
+            self.currentSynapse = get_str_attribute_group(g,'synapse')
         elif "postComponent" in g._v_attrs:
-            self.currentSynapse = g._v_attrs.postComponent
+            self.currentSynapse = get_str_attribute_group(g,'postComponent')
             
         if "preComponent" in g._v_attrs:
-            self.currentPreSynapse = g._v_attrs.preComponent
+            self.currentPreSynapse = get_str_attribute_group(g,'preComponent')
           
         
         if not self.optimized:
-            self.log.debug("------    Found a projection: "+ self.currentProjectionId+ ", from "+ self.currentProjectionPrePop
-            +" to "+ self.currentProjectionPostPop+" through "+self.currentSynapse)
+            self.log.debug("------    Found a projection: %s, from %s to %s through %s" %(self.currentProjectionId,self.currentProjectionPrePop,self.currentProjectionPostPop,self.currentSynapse))
         else:
             
             if self.currentProjectionType=='electricalProjection':           
@@ -517,9 +558,9 @@ class NeuroMLHdf5Parser():
     
     if g._v_name.count('inputList_')>=1 or g._v_name.count('input_list_')>=1: # inputList_ preferred
         # TODO: a better check to see if the attribute is a str or numpy.ndarray
-        self.currInputList = g._v_attrs.id
-        component = g._v_attrs.component
-        population = g._v_attrs.population
+        self.currInputList = get_str_attribute_group(g,'id')
+        component = get_str_attribute_group(g,'component')
+        population = get_str_attribute_group(g,'population')
         
         size = self._get_node_size(g,self.currInputList)
         
@@ -572,6 +613,7 @@ if __name__ == '__main__':
 
 
     file_name = '../examples/test_files/complete.nml.h5'
+    #file_name = '../../../neuroConstruct/osb/showcase/NetPyNEShowcase/NeuroML2/scaling/Balanced.net.nml.h5'
 
     logging.basicConfig(level=logging.DEBUG, format="%(name)-19s %(levelname)-5s - %(message)s")
 
