@@ -12,6 +12,7 @@
 #
   
 import logging
+import inspect
 
 class NeuroMLXMLParser():
     
@@ -31,6 +32,33 @@ class NeuroMLXMLParser():
     
     self.netHandler = netHandler
     
+    # For continued use with old API
+    if not hasattr(self.netHandler,'handle_network') or hasattr(self.netHandler,'handleNetwork'):
+        self.netHandler.handle_network = self.netHandler.handleNetwork
+    if not hasattr(self.netHandler,'handle_document_start') or hasattr(self.netHandler,'handleDocumentStart'):
+        self.netHandler.handle_document_start = self.netHandler.handleDocumentStart
+    if not hasattr(self.netHandler,'handle_population') or hasattr(self.netHandler,'handlePopulation'):
+        self.netHandler.handle_population = self.netHandler.handlePopulation
+    if not hasattr(self.netHandler,'handle_location') or hasattr(self.netHandler,'handleLocation'):
+        self.netHandler.handle_location = self.netHandler.handleLocation
+        
+    if not hasattr(self.netHandler,'handle_projection') or hasattr(self.netHandler,'handleProjection'):
+        self.netHandler.handle_projection = self.netHandler.handleProjection
+    if not hasattr(self.netHandler,'finalise_projection') or hasattr(self.netHandler,'finaliseProjection'):
+        self.netHandler.finalise_projection = self.netHandler.finaliseProjection
+        
+    if not hasattr(self.netHandler,'handle_connection') or hasattr(self.netHandler,'handleConnection'):
+        self.netHandler.handle_connection = self.netHandler.handleConnection
+    if not hasattr(self.netHandler,'handle_input_list') or hasattr(self.netHandler,'handleInputList'):
+        self.netHandler.handle_input_list = self.netHandler.handleInputList
+    if not hasattr(self.netHandler,'handle_single_input') or hasattr(self.netHandler,'handleSingleInput'):
+        self.netHandler.handle_single_input = self.netHandler.handleSingleInput
+    if not hasattr(self.netHandler,'finalise_input_source') or hasattr(self.netHandler,'finaliseInputSource'):
+        self.netHandler.finalise_input_source = self.netHandler.finaliseInputSource
+        
+    
+        
+        
   def _parse_delay(self, delay_string):
       if delay_string.endswith('ms'):
           return float(delay_string[:-2].strip())
@@ -67,11 +95,17 @@ class NeuroMLXMLParser():
             
             if len(population.instances)>0 and population.type=='populationList':
                   
-                self.netHandler.handle_population(population.id, 
-                                                 population.component, 
-                                                 len(population.instances),
-                                                 component_obj=component_obj,
-                                                 properties=properties)
+                if 'properties' in inspect.getargspec(self.netHandler.handle_population)[0]:
+                    self.netHandler.handle_population(population.id, 
+                                                     population.component, 
+                                                     len(population.instances),
+                                                     component_obj=component_obj,
+                                                     properties=properties)
+                else:
+                    self.netHandler.handle_population(population.id, 
+                                                     population.component, 
+                                                     len(population.instances),
+                                                     component_obj=component_obj)
 
                 for inst in population.instances:
 
@@ -83,10 +117,18 @@ class NeuroMLXMLParser():
                                             loc.y,       \
                                             loc.z)       
             else:
-                self.netHandler.handle_population(population.id, 
-                                                 population.component, 
-                                                 population.size,
-                                                 component_obj=component_obj)
+                
+                if 'properties' in inspect.getargspec(self.netHandler.handle_population)[0]:
+                    self.netHandler.handle_population(population.id, 
+                                                     population.component, 
+                                                     population.size,
+                                                     component_obj=component_obj,
+                                                     properties=properties)
+                else:
+                    self.netHandler.handle_population(population.id, 
+                                                     population.component, 
+                                                     population.size,
+                                                     component_obj=component_obj)
                                                      
                 for i in range(population.size):
                     self.netHandler.handle_location(i,                      \
@@ -213,6 +255,7 @@ class NeuroMLXMLParser():
             pre_comp = None
             post_comp = None
             
+            
             for connection in cp.continuous_connections:
                 if pre_comp != None and pre_comp != connection.pre_component:
                     raise Exception("There are different pre components for connections inside: %s!"%cp)
@@ -240,6 +283,8 @@ class NeuroMLXMLParser():
             pre_obj = self.nml_doc.get_by_id(pre_comp)
             post_obj = self.nml_doc.get_by_id(post_comp)
             
+            synapse = pre_comp
+            
             self.netHandler.handle_projection(cp.id,
                                             cp.presynaptic_population,
                                             cp.postsynaptic_population,
@@ -313,17 +358,27 @@ class NeuroMLXMLParser():
                                                   segId = input_w.get_segment_id(), 
                                                   fract = input_w.get_fraction_along(),
                                                   weight = input_w.get_weight())
+                                                  
+            self.netHandler.finalise_input_source(input_list.id)
                                                
         for explicitInput in network.explicit_inputs:     
+            
             list_name = 'INPUT_%s_%s'%(explicitInput.input,explicitInput.target.replace('[','_').replace(']','_'))
+            if list_name.endswith('_'): list_name=list_name[:-1]
+            
             pop = explicitInput.target.split('[')[0]
-            self.netHandler.handle_input_list(list_name, pop, explicitInput.input, 1)   
+            
+            input_comp_obj = self.nml_doc.get_by_id(explicitInput.input)
+            
+            self.netHandler.handle_input_list(list_name, pop, explicitInput.input, 1,input_comp_obj=input_comp_obj)   
 
             self.netHandler.handle_single_input(list_name, 
                                               0, 
                                               cellId = explicitInput.get_target_cell_id(), 
                                               segId = 0, 
                                               fract = 0.5)
+                                              
+            self.netHandler.finalise_input_source(list_name)
     
         
         
