@@ -15,12 +15,16 @@ except ImportError:
 class TestNeuroMLLoader(unittest.TestCase):
     def test_load_neuroml(self):
         root_dir = os.path.dirname(neuroml.__file__)
-        print('root dir is:')
-        print(root_dir)
+        print('root dir is: '+root_dir)
         test_file_path = os.path.join(root_dir,'examples/test_files/Purk2M9s.nml')
-        print('test file path is:')
-        print(test_file_path)
+        print('test file path is: ' + test_file_path)
         f = open(test_file_path,'r')
         #print(f.read())
         doc = loaders.NeuroMLLoader.load(test_file_path)
         self.assertEqual(doc.id,'Purk2M9s')
+        f.close()
+        print('Finished test')
+
+if __name__ == "__main__":
+    t = TestNeuroMLLoader()
+    t.test_load_neuroml()
\ No newline at end of file
